@@ -52,7 +52,17 @@ namespace mapnik {
         swap(tmp);
         return *this;
     }
-        
+    
+    bool projection::operator==(const projection& other) const 
+    {
+        return (params_ == other.params_);
+    }
+    
+    bool projection::operator!=(const projection& other) const 
+    {
+        return !(*this == other);
+    }
+    
     bool projection::is_initialized() const
     {
         return proj_ ? true : false;
@@ -63,7 +73,7 @@ namespace mapnik {
 #ifdef MAPNIK_THREADSAFE
         mutex::scoped_lock lock(mutex_);
 #endif
-        return pj_is_latlong(proj_);  
+        return pj_is_latlong(proj_) ? true : false;  
     }
     
     std::string const& projection::params() const
@@ -82,6 +92,11 @@ namespace mapnik {
         p = pj_fwd(p,proj_);
         x = p.u;
         y = p.v;
+        if (pj_is_latlong(proj_))
+        {
+           x *=RAD_TO_DEG;
+           y *=RAD_TO_DEG;
+        }           
     }
     
     void projection::inverse(double & x,double & y) const
@@ -89,12 +104,17 @@ namespace mapnik {
 #ifdef MAPNIK_THREADSAFE
        mutex::scoped_lock lock(mutex_);
 #endif
-        projUV p;
-        p.u = x;
-        p.v = y;
-        p = pj_inv(p,proj_);
-        x = RAD_TO_DEG * p.u;
-        y = RAD_TO_DEG * p.v;
+       if (pj_is_latlong(proj_))
+       {
+          x *=DEG_TO_RAD;
+          y *=DEG_TO_RAD;
+       }  
+       projUV p;
+       p.u = x;
+       p.v = y;
+       p = pj_inv(p,proj_);
+       x = RAD_TO_DEG * p.u;
+       y = RAD_TO_DEG * p.v;
     }
     
     projection::~projection() 
